@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,6 +55,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.realtime.group.GroupCommand;
 import com.openexchange.realtime.group.commands.JoinCommand;
 import com.openexchange.realtime.group.commands.LeaveCommand;
+import com.openexchange.realtime.group.commands.PingCommand;
 import com.openexchange.tools.session.ServerSession;
 
 
@@ -88,6 +89,9 @@ public class GroupCommand2JSON implements SimplePayloadConverter {
         if (LeaveCommand.class.isInstance(data)) {
             return "leave";
         }
+        if (PingCommand.class.isInstance(data)) {
+            return "ping";
+        }
         return "unknown: " + data.getClass().getSimpleName();
     }
 
