@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,22 +51,32 @@ package com.openexchange.realtime.osgi;
 
 import java.util.concurrent.TimeUnit;
 import org.osgi.framework.ServiceReference;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.context.ContextService;
 import com.openexchange.conversion.simple.SimpleConverter;
+import com.openexchange.conversion.simple.SimplePayloadConverter;
+import com.openexchange.exception.OXException;
 import com.openexchange.management.ManagementService;
 import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.osgi.SimpleRegistryListener;
 import com.openexchange.realtime.Channel;
 import com.openexchange.realtime.Component;
 import com.openexchange.realtime.RealtimeConfig;
+import com.openexchange.realtime.cleanup.GlobalRealtimeCleanup;
 import com.openexchange.realtime.cleanup.LocalRealtimeCleanup;
 import com.openexchange.realtime.cleanup.LocalRealtimeCleanupImpl;
+import com.openexchange.realtime.cleanup.RealtimeJanitor;
 import com.openexchange.realtime.management.ManagementHouseKeeper;
+import com.openexchange.realtime.packet.ID;
+import com.openexchange.realtime.packet.IDManager;
 import com.openexchange.realtime.payload.PayloadTree;
 import com.openexchange.realtime.payload.PayloadTreeNode;
 import com.openexchange.realtime.payload.converter.PayloadTreeConverter;
 import com.openexchange.realtime.payload.converter.impl.DefaultPayloadTreeConverter;
+import com.openexchange.realtime.payload.converter.impl.DurationToJSONConverter;
+import com.openexchange.realtime.payload.converter.impl.JSONToDurationConverter;
 import com.openexchange.realtime.synthetic.DevNullChannel;
 import com.openexchange.realtime.synthetic.SyntheticChannel;
 import com.openexchange.threadpool.ThreadPoolService;
@@ -81,6 +91,7 @@ import com.openexchange.user.UserService;
  */
 public class RealtimeActivator extends HousekeepingActivator {
 
+    private static final Logger LOG = LoggerFactory.getLogger(RealtimeActivator.class);
     private SyntheticChannel synth;
     private RealtimeConfig realtimeConfig;
 
@@ -99,8 +110,30 @@ public class RealtimeActivator extends HousekeepingActivator {
         realtimeConfig.start();
 
         managementHouseKeeper.addManagementObject(realtimeConfig.getManagementObject());
+        
+        IDManager idManager = new IDManager();
+        ID.ID_MANAGER_REF.set(idManager);
+        RealtimeJanitors.getInstance().addJanitor(idManager);
 
-        synth = new SyntheticChannel(this);
+        //Add the node-wide cleanup service and start tracking Janitors 
+        LocalRealtimeCleanupImpl localRealtimeCleanup = new LocalRealtimeCleanupImpl(context);
+        rememberTracker(localRealtimeCleanup);
+        registerService(LocalRealtimeCleanup.class, localRealtimeCleanup);
+        //And track the cluster wide cleanup
+        track(GlobalRealtimeCleanup.class, new SimpleRegistryListener<GlobalRealtimeCleanup>() {
+
+            @Override
+            public void added(ServiceReference<GlobalRealtimeCleanup> ref, GlobalRealtimeCleanup service) {
+                SyntheticChannel.GLOBAL_CLEANUP_REF.set(service);
+            }
+
+            @Override
+            public void removed(ServiceReference<GlobalRealtimeCleanup> ref, GlobalRealtimeCleanup service) {
+                SyntheticChannel.GLOBAL_CLEANUP_REF.set(null);
+            }
+        });
+        
+        synth = new SyntheticChannel(this, localRealtimeCleanup);
 
         TimerService timerService = getService(TimerService.class);
         timerService.scheduleAtFixedRate(synth, 0, 1, TimeUnit.MINUTES);
@@ -127,14 +160,20 @@ public class RealtimeActivator extends HousekeepingActivator {
         registerService(PayloadTreeConverter.class, converter);
 
         registerService(Channel.class, new DevNullChannel());
+        registerService(SimplePayloadConverter.class, new DurationToJSONConverter());
+        registerService(SimplePayloadConverter.class, new JSONToDurationConverter());
 
-        //Add the node-wide cleanup service and start tracking Janitors 
-        LocalRealtimeCleanupImpl localRealtimeCleanup = new LocalRealtimeCleanupImpl(context);
-        rememberTracker(localRealtimeCleanup);
-        registerService(LocalRealtimeCleanup.class, localRealtimeCleanup);
+        //Register all RealtimeJanitors
+        for(RealtimeJanitor realtimeJanitor : RealtimeJanitors.getInstance().getJanitors()) {
+            registerService(RealtimeJanitor.class, realtimeJanitor);
+        }
 
         //Expose all ManagementObjects for this bundle
-        managementHouseKeeper.exposeManagementObjects();
+        try {
+            managementHouseKeeper.exposeManagementObjects();
+        } catch (OXException oxe) {
+            LOG.error("Failed to expose ManagementObjects", oxe);
+        }
         openTrackers();
     }
 
@@ -143,6 +182,8 @@ public class RealtimeActivator extends HousekeepingActivator {
         synth.shutdown();
         //Conceal all ManagementObjects for this bundle and remove them from the housekeeper
         ManagementHouseKeeper.getInstance().cleanup();
+        ID.ID_MANAGER_REF.set(null);
+        RealtimeJanitors.getInstance().cleanup();
         realtimeConfig.stop();
         super.cleanUp();
     }
